#   DrCr: Web-based double-entry bookkeeping framework
#   Copyright (C) 2022  Lee Yingtong Li (RunasSudo)
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

from . import AMOUNT_DPS

class Transaction:
	def __init__(self, dt=None, description=None, postings=None):
		self.dt = dt
		self.description = description
		self.postings = postings or []
	
	def assert_valid(self):
		"""Assert that debits equal credits, and commodities are compatible"""
		
		if any(p.commodity != '$' for p in self.postings):
			# FIXME: Allow non-$ commodities
			raise AssertionError('Transaction contains multiple commodities')
		
		if sum(p.quantity for p in self.postings) != 0:
			raise AssertionError('Transaction debits and credits do not balance')

class Posting:
	def __init__(self, description=None, account=None, quantity=None, commodity=None):
		self.description = description
		self.account = account
		self.quantity = quantity
		self.commodity = commodity
	
	def amount(self):
		return Amount(self.quantity, self.commodity)

class Amount:
	__slots__ = ['quantity', 'commodity']
	
	def __init__(self, quantity, commodity):
		self.quantity = quantity
		self.commodity = commodity
	
	def __abs__(self):
		return Amount(abs(self.quantity), self.commodity)
	
	def format(self):
		return '{0}{1:,.{dps}f}'.format(self.commodity, self.quantity / (10**AMOUNT_DPS), dps=AMOUNT_DPS)
	
	def quantity_string(self):
		return '{:.{dps}f}'.format(self.quantity / (10**AMOUNT_DPS), dps=AMOUNT_DPS)
